@@ -218,9 +218,10 @@ public:
     /**
      * Set the local network address of the RTP transport
      * @param addr New local RTP transport address
+     * @param rtcp Enable RTCP transport
      * @return True if address set, false if a failure occured
      */
-    bool localAddr(SocketAddr& addr);
+    bool localAddr(SocketAddr& addr, bool rtcp = true);
 
     /**
      * Set the remote network address of the RTP transport
@@ -333,7 +334,7 @@ public:
      * Default constructor.
      */
     inline RTPBaseIO(RTPSession* session = 0)
-	: m_session(session), m_ssrc(0), m_ts(0), m_seq(0),
+	: m_session(session), m_ssrcInit(true), m_ssrc(0), m_ts(0), m_seq(0),
 	  m_evTs(0), m_evNum(-1), m_evVol(-1),
 	  m_dataType(-1), m_eventType(-1), m_silenceType(-1)
 	{ }
@@ -388,22 +389,22 @@ public:
     bool silencePayload(int type);
 
     /**
-     * Reset the SSRC requesting generation/grabbing of a new one
+     * Requesting generation/grabbing of a new SSRC
      */
     inline void reset()
-	{ m_ssrc = 0; }
+	{ m_ssrcInit = true; }
 
     /**
      * Get the value of the current SSRC, zero if not initialized yet
      */
     inline unsigned int ssrc() const
-	{ return m_ssrc; }
+	{ return m_ssrcInit ? 0 : m_ssrc; }
 
     /**
      * Force a new known SSRC for all further packets
      */
     inline void ssrc(unsigned int src)
-	{ m_ssrc = src; }
+	{ m_ssrc = src; m_ssrcInit = false; }
 
 protected:
     /**
@@ -413,6 +414,7 @@ protected:
     virtual void timerTick(const Time& when) = 0;
 
     RTPSession* m_session;
+    bool m_ssrcInit;
     u_int32_t m_ssrc;
     u_int32_t m_ts;
     u_int16_t m_seq;
@@ -541,10 +543,10 @@ class YRTP_API RTPSender : public RTPBaseIO
 public:
     /**
      * Constructor
+     * @param session RTP session the sender belongs
+     * @param randomTs Initialize a random timestamp offset
      */
-    inline RTPSender(RTPSession* session = 0)
-	: RTPBaseIO(session), m_evTime(0), m_tsLast(0)
-	{ }
+    RTPSender(RTPSession* session = 0, bool randomTs = true);
 
     /**
      * Do-nothing destructor
@@ -873,10 +875,11 @@ public:
     /**
      * Set the local network address of the RTP transport of this session
      * @param addr New local RTP transport address
+     * @param rtpc Enable RTCP in this session
      * @return True if address set, false if a failure occured
      */
-    inline bool localAddr(SocketAddr& addr)
-	{ return m_transport && m_transport->localAddr(addr); }
+    inline bool localAddr(SocketAddr& addr, bool rtcp = true)
+	{ return m_transport && m_transport->localAddr(addr,rtcp); }
 
     /**
      * Set the remote network address of the RTP transport of this session
