@@ -585,6 +585,12 @@ void Channel::callAccept(Message& msg)
     if (m_billid.null())
 	m_billid = msg.getValue("billid");
     m_targetid = msg.getValue("targetid");
+    String detect = msg.getValue("tonedetect_in");
+    if (detect && detect.toBoolean(true)) {
+	if (detect.toBoolean(false))
+	    detect = "tone/*";
+	toneDetect(detect);
+    }
     if (msg.getBoolValue("autoanswer"))
 	msgAnswered(msg);
     else if (msg.getBoolValue("autoring"))
@@ -599,6 +605,16 @@ void Channel::callAccept(Message& msg)
     }
 }
 
+void Channel::callConnect(Message& msg)
+{
+    String detect = msg.getValue("tonedetect_out");
+    if (detect && detect.toBoolean(true)) {
+	if (detect.toBoolean(false))
+	    detect = "tone/*";
+	toneDetect(detect);
+    }
+}
+
 void Channel::callRejected(const char* error, const char* reason, const Message* msg)
 {
     Debug(this,DebugMild,"Call rejected error='%s' reason='%s' [%p]",error,reason,this);
@@ -619,6 +635,18 @@ bool Channel::dtmfInband(const char* tone)
     return Engine::dispatch(m);
 }
 
+bool Channel::toneDetect(const char* sniffer)
+{
+    if (null(sniffer))
+	return false;
+    Message m("chan.attach");
+    complete(m,true);
+    m.userData(this);
+    m.setParam("sniffer",sniffer);
+    m.setParam("single","yes");
+    return Engine::dispatch(m);
+}
+
 bool Channel::setDebug(Message& msg)
 {
     String str = msg.getValue("line");
