@@ -55,6 +55,15 @@ public:
     virtual bool received(Message &msg);
 };
 
+class UIUserNotifyHandler : public MessageHandler
+{
+public:
+    UIUserNotifyHandler()
+	: MessageHandler("user.notify",50)
+	{ }
+    virtual bool received(Message &msg);
+};
+
 class ClientThreadProxy
 {
 public:
@@ -379,6 +388,7 @@ static Configuration s_accounts;
 static Configuration s_contacts;
 static Configuration s_providers;
 static Configuration s_history;
+static unsigned int s_eventLen = 0;
 
 // Parameters that are stored with account
 static const char* s_accParams[] = {
@@ -406,6 +416,7 @@ Client::Client(const char *name)
     s_client = this;
     Engine::install(new UICdrHandler);
     Engine::install(new UIUserHandler);
+    Engine::install(new UIUserNotifyHandler);
     Engine::install(new UIHandler);
 }
 
@@ -425,11 +436,12 @@ void Client::run()
     initWindows();
     initClient();
     updateFrom(0);
-    setStatus(Engine::config().getValue("client","greeting","Yate " YATE_VERSION " " YATE_RELEASE));
+    setStatus(Engine::config().getValue("client","greeting","Yate " YATE_VERSION " - " YATE_RELEASE));
     m_initialized = true;
     msg.setParam("event","init");
     Engine::dispatch(msg);
     main();
+    exitClient();
 }
 
 Window* Client::getWindow(const String& name)
@@ -487,6 +499,12 @@ void Client::initWindows()
 
 void Client::initClient()
 {
+    s_eventLen = Engine::config().getIntValue("client","eventlen",4096);
+    if (s_eventLen > 65535)
+	s_eventLen = 65535;
+    else if (s_eventLen && (s_eventLen < 1024))
+	s_eventLen = 1024;
+
     s_accounts = Engine::configFile("client_accounts",true);
     s_accounts.load();
     unsigned int n = s_accounts.sections();
@@ -540,20 +558,47 @@ void Client::initClient()
 	    updateCallHist(*sect);
     }
 
+    Configuration settings(Engine::configFile("client_settings",true));
+    settings.load();
+    // guess if we at least can support multiple lines
     bool tmp =
 	getWindow("channels") || hasElement("channels") ||
 	getWindow("lines") || hasElement("lines");
+    // then apply saved preferences
+    tmp = settings.getBoolValue("general","multilines",tmp);
+    // finally restrict from the main config file
     m_multiLines = Engine::config().getBoolValue("client","multilines",tmp);
     tmp = false;
     getCheck("autoanswer",tmp);
+    tmp = settings.getBoolValue("general","autoanswer",tmp);
     m_autoAnswer = Engine::config().getBoolValue("client","autoanswer",tmp);
     setCheck("multilines",m_multiLines);
     setCheck("autoanswer",m_autoAnswer);
+    setText("def_username",settings.getValue("default","username"));
+    setText("def_callerid",settings.getValue("default","callerid"));
+    setText("def_domain",settings.getValue("default","domain"));
     Window* help = getWindow("help");
     if (help)
 	action(help,"help_home");
 }
 
+void Client::exitClient()
+{
+    Configuration settings(Engine::configFile("client_settings",true));
+    settings.setValue("general","multilines",m_multiLines);
+    settings.setValue("general","autoanswer",m_autoAnswer);
+    String tmp;
+    if (getText("def_username",tmp))
+	settings.setValue("default","username",tmp);
+    tmp.clear();
+    if (getText("def_callerid",tmp))
+	settings.setValue("default","callerid",tmp);
+    tmp.clear();
+    if (getText("def_domain",tmp))
+	settings.setValue("default","domain",tmp);
+    settings.save();
+}
+
 void Client::moveRelated(const Window* wnd, int dx, int dy)
 {
     if (!wnd)
@@ -966,10 +1011,36 @@ bool Client::getSelect(const String& name, String& item, Window* wnd, Window* sk
     return false;
 }
 
+bool Client::addToLog(const String& text, Window* wnd)
+{
+    String tmp;
+    if (getText("log_events",tmp,wnd)) {
+	tmp.append(text,"\n");
+	while (s_eventLen && (tmp.length() > s_eventLen)) {
+	    int pos = tmp.find('\n');
+	    if (pos < 0)
+		break;
+	    tmp.assign(tmp.c_str()+pos+1);
+	}
+	setText("log_events",tmp,wnd);
+	return true;
+    }
+    return false;
+}
+
 bool Client::setStatus(const String& text, Window* wnd)
 {
     Debug(ClientDriver::self(),DebugInfo,"Status '%s' in window %p",text.c_str(),wnd);
-    return setText("status",text,wnd);
+    bool ok = addToLog(text,wnd);
+    return setText("status",text,wnd) || ok;
+}
+
+bool Client::addToLogLocked(const String& text, Window* wnd)
+{
+    lockOther();
+    bool ok = addToLog(text,wnd);
+    unlockOther();
+    return ok;
 }
 
 bool Client::setStatusLocked(const String& text, Window* wnd)
@@ -1170,9 +1241,13 @@ bool Client::action(Window* wnd, const String& name)
 		String name;
 		name << "acc_" << *par;
 		String val;
-		if (getText(name,val,wnd) && val) {
-		    s_accounts.setValue(newAcc,*par,val);
-		    m->addParam(*par,val);
+		if (getText(name,val,wnd)) {
+		    if (val.null())
+			s_accounts.clearKey(newAcc,*par);
+		    else {
+			s_accounts.setValue(newAcc,*par,val);
+			m->addParam(*par,val);
+		    }
 		}
 	    }
 	    Engine::enqueue(m);
@@ -1363,6 +1438,13 @@ bool Client::action(Window* wnd, const String& name)
 	    return true;
 	}
     }
+    // event log actions
+    else if (name == "log_events_clear") {
+	// clear the event log, be it table or text
+	bool ok = clearTable("log_events");
+	ok = setText("log_events","") || ok;
+	return ok;
+    }
     // help window actions
     else if (wnd && name.startsWith("help_")) {
 	bool show = false;
@@ -1867,6 +1949,8 @@ bool UIHandler::received(Message &msg)
     Window* wnd = Client::getWindow(msg.getValue("window"));
     if (action == "set_status")
 	return Client::self()->setStatusLocked(msg.getValue("status"),wnd);
+    else if (action == "add_log")
+	return Client::self()->addToLogLocked(msg.getValue("text"),wnd);
     else if (action == "show_message") {
 	Client::self()->lockOther();
 	bool ok = Client::openMessage(msg.getValue("text"),Client::getWindow(msg.getValue("parent")),msg.getValue("context"));
@@ -1960,6 +2044,32 @@ bool UIUserHandler::received(Message &msg)
 }
 
 
+bool UIUserNotifyHandler::received(Message &msg)
+{
+    if (!Client::self())
+	return false;
+    String account = msg.getValue("account");
+    if (account.null())
+	return false;
+    bool reg = msg.getBoolValue("registered");
+    const char* proto = msg.getValue("protocol");
+    const char* reason = msg.getValue("reason");
+    String txt = reg ? "Registered" : "Unregistered";
+    if (proto)
+	txt << " " << proto;
+    txt << " account " << account;
+    if (reason)
+	txt << " reason: " << reason;
+
+    // block until client finishes initialization
+    while (!Client::self()->initialized())
+	Thread::msleep(10);
+
+    Client::self()->addToLogLocked(txt);
+    return false;
+}
+
+
 // IMPORTANT: having a target means "from inside Yate to the user"
 //  An user initiated call must be incoming (no target)
 ClientChannel::ClientChannel(const String& party, const char* target, const Message* msg)
@@ -2089,7 +2199,7 @@ bool ClientChannel::callRouted(Message& msg)
 void ClientChannel::callAccept(Message& msg)
 {
     Debug(ClientDriver::self(),DebugAll,"ClientChannel::callAccept() [%p]",this);
-    Client::self()->setStatusLocked("Call connected");
+    Client::self()->setStatusLocked("Calling target");
     Channel::callAccept(msg);
     update();
 }
