@@ -485,12 +485,19 @@ void DataSource::Forward(const DataBlock& data, unsigned long tStamp)
 	DDebug(DebugInfo,"Forwarding on a dead DataSource! [%p]",this);
 	return;
     }
-    // no timestamp provided - try to guess
-    if (tStamp == (unsigned long)-1) {
-	tStamp = m_timestamp;
-	const FormatInfo* f = m_format.getInfo();
-	if (f)
-	    tStamp += f->guessSamples(data.length());
+
+    // try to evaluate amount of samples in this packet
+    const FormatInfo* f = m_format.getInfo();
+    unsigned long nSamp = f ? f->guessSamples(data.length()) : 0;
+
+    // if no timestamp provided - try to use next expected
+    if (tStamp == invalidStamp())
+	tStamp = m_nextStamp;
+    // still no timestamp known - wild guess based on this packet size
+    if (tStamp == invalidStamp()) {
+	DDebug(DebugNote,"Unknow timestamp - assuming %lu + %lu [%p]",
+	    m_timestamp,nSamp,this);
+	tStamp = m_timestamp + nSamp;
     }
     ObjList *l = m_consumers.skipNull();
     for (; l; l=l->skipNext()) {
@@ -498,6 +505,7 @@ void DataSource::Forward(const DataBlock& data, unsigned long tStamp)
 	c->Consume(data,tStamp,this);
     }
     m_timestamp = tStamp;
+    m_nextStamp = nSamp ? (tStamp + nSamp) : invalidStamp();
 }
 
 bool DataSource::attach(DataConsumer* consumer, bool override)
@@ -582,6 +590,7 @@ void DataSource::synchronize(unsigned long tStamp)
 	return;
     }
     m_timestamp = tStamp;
+    m_nextStamp = invalidStamp();
     ObjList *l = m_consumers.skipNull();
     for (; l; l=l->skipNext()) {
 	DataConsumer *c = static_cast<DataConsumer *>(l->get());
@@ -616,6 +625,7 @@ DataEndpoint::~DataEndpoint()
     disconnect();
     setPeerRecord();
     setCallRecord();
+    clearSniffers();
     setSource();
     setConsumer();
 }
@@ -748,6 +758,9 @@ void DataEndpoint::setSource(DataSource* source)
 	    if (m_callRecord->getConnSource())
 		Debug(DebugWarn,"consumer source not cleared in %p",m_callRecord);
 	}
+	ObjList* l = m_sniffers.skipNull();
+	for (; l; l = l->skipNext())
+	    DataTranslator::detachChain(temp,static_cast<DataConsumer*>(l->get()));
 	temp->deref();
     }
     if (source) {
@@ -758,6 +771,9 @@ void DataEndpoint::setSource(DataSource* source)
 	    DataTranslator::attachChain(source,c2);
 	if (m_callRecord)
 	    DataTranslator::attachChain(source,m_callRecord);
+	ObjList* l = m_sniffers.skipNull();
+	for (; l; l = l->skipNext())
+	    DataTranslator::attachChain(source,static_cast<DataConsumer*>(l->get()));
     }
     m_source = source;
     if (c1)
@@ -842,6 +858,53 @@ void DataEndpoint::setCallRecord(DataConsumer* consumer)
     }
 }
 
+bool DataEndpoint::addSniffer(DataConsumer* sniffer)
+{
+    if (!sniffer)
+	return false;
+    Lock lock(s_dataMutex);
+    if (m_sniffers.find(sniffer))
+	return false;
+    if (!sniffer->ref())
+	return false;
+    XDebug(DebugInfo,"DataEndpoint::addSniffer(%p) s=%p [%p]",
+	sniffer,m_source,this);
+    m_sniffers.append(sniffer);
+    if (m_source)
+	DataTranslator::attachChain(m_source,sniffer);
+    return true;
+}
+
+bool DataEndpoint::delSniffer(DataConsumer* sniffer)
+{
+    if (!sniffer)
+	return false;
+    Lock lock(s_dataMutex);
+    XDebug(DebugInfo,"DataEndpoint::delSniffer(%p) s=%p [%p]",
+	sniffer,m_source,this);
+    if (!m_sniffers.remove(sniffer,false))
+	return false;
+    if (m_source)
+	DataTranslator::detachChain(m_source,sniffer);
+    sniffer->deref();
+    return true;
+}
+
+void DataEndpoint::clearSniffers()
+{
+    Lock lock(s_dataMutex);
+    for (;;) {
+	DataConsumer* sniffer = static_cast<DataConsumer*>(m_sniffers.remove(false));
+	if (!sniffer)
+	    return;
+	XDebug(DebugInfo,"DataEndpoint::clearSniffers() sn=%p s=%p [%p]",
+	    sniffer,m_source,this);
+	if (m_source)
+	    DataTranslator::detachChain(m_source,sniffer);
+	sniffer->deref();
+    }
+}
+
 
 ThreadedSource::~ThreadedSource()
 {
