@@ -247,6 +247,7 @@ public:
     virtual void initialize();
     virtual bool msgExecute(Message& msg, String& dest);
     virtual bool msgRoute(Message& msg);
+    virtual void msgTimer(Message& msg);
     virtual bool received(Message &msg, int id);
     virtual void statusParams(String& str);
     void cleanup();
@@ -349,10 +350,13 @@ public:
     bool startGkClient(int mode, int retry = 0, const char* name = "");
     void stopGkClient();
     void asyncGkClient(int mode, const PString& name, int retry);
+    void checkGkClient();
 protected:
     bool internalGkClient(int mode, const PString& name);
+    void internalGkNotify(bool registered);
     YateGatekeeperServer* m_gkServer;
     YateGkRegThread* m_thread;
+    bool m_registered;
 };
 
 class YateH323Connection :  public H323Connection, public DebugEnabler
@@ -460,6 +464,7 @@ public:
     virtual void callAccept(Message& msg);
     virtual void callRejected(const char* error, const char* reason, const Message* msg);
     virtual bool setDebug(Message& msg);
+    void setAddress(const char* addr);
     inline void setTarget(const char* targetid)
 	{ m_targetid = targetid; }
 private:
@@ -655,7 +660,7 @@ BOOL YateGatekeeperServer::Init()
 }
 
 YateH323EndPoint::YateH323EndPoint(const NamedList* params, const char* name)
-    : String(name), m_gkServer(0), m_thread(0)
+    : String(name), m_gkServer(0), m_thread(0), m_registered(false)
 {
     Debug(&hplugin,DebugAll,"YateH323EndPoint::YateH323EndPoint(%p,\"%s\") [%p]",
 	params,name,this);
@@ -777,7 +782,9 @@ bool YateH323EndPoint::Init(const NamedList* params)
 	SetLocalUserName(ali);
 	const char* server = params ? params->getValue("server") : 0;
 	if (params && params->getBoolValue("gkclient",server != 0)) {
-	    int ttl = params->getIntValue("gkttl",300);
+	    int ttl = params->getIntValue("interval",300);
+	    // "gkttl" is deprecated
+	    ttl = params->getIntValue("gkttl",ttl);
 	    if (ttl > 0) {
 		// adjust time to live between 1 minute and 1 day
 		if (ttl < 60)
@@ -792,7 +799,7 @@ bool YateH323EndPoint::Init(const NamedList* params)
 	    const char *p = params->getValue("password");
 	    if (p) {
 		SetGatekeeperPassword(p);
-		Debug(&hplugin,DebugInfo,"Enabling H.235 security access to gatekeeper %s",p);
+		DDebug(&hplugin,DebugInfo,"Enabling H.235 security access to gatekeeper: '%s'",p);
 	    }
 	    const char* d = params->getValue("gkip",server);
 	    const char* a = params->getValue("gkname");
@@ -845,12 +852,14 @@ void YateH323EndPoint::stopGkClient()
 	lock.drop();
 	RemoveGatekeeper();
     }
+    internalGkNotify(false);
 }
 
 void YateH323EndPoint::asyncGkClient(int mode, const PString& name, int retry)
 {
     while (!internalGkClient(mode,name) && (retry > 0))
 	Thread::sleep(retry);
+    Debug(&hplugin,DebugNote,"Thread for GK client '%s' finished",(const char*)name);
     hplugin.lock();
     m_thread = 0;
     hplugin.unlock();
@@ -863,6 +872,7 @@ bool YateH323EndPoint::internalGkClient(int mode, const PString& name)
 	    if (SetGatekeeper(name,new H323TransportUDP(*this))) {
 		Debug(&hplugin,DebugInfo,"Connected '%s' to GK addr '%s'",
 		    safe(),(const char*)name);
+		internalGkNotify(true);
 		return true;
 	    }
 	    Debug(&hplugin,DebugWarn,"Failed to connect '%s' to GK addr '%s'",
@@ -872,6 +882,7 @@ bool YateH323EndPoint::internalGkClient(int mode, const PString& name)
 	    if (LocateGatekeeper(name)) {
 		Debug(&hplugin,DebugInfo,"Connected '%s' to GK name '%s'",
 		    safe(),(const char*)name);
+		internalGkNotify(true);
 		return true;
 	    }
 	    Debug(&hplugin,DebugWarn,"Failed to connect '%s' to GK name '%s'",
@@ -880,6 +891,7 @@ bool YateH323EndPoint::internalGkClient(int mode, const PString& name)
 	case Discover:
 	    if (DiscoverGatekeeper(new H323TransportUDP(*this))) {
 		Debug(&hplugin,DebugInfo,"Connected '%s' to discovered GK",safe());
+		internalGkNotify(true);
 		return true;
 	    }
 	    Debug(&hplugin,DebugWarn,"Failed to discover a GK in '%s'",safe());
@@ -887,11 +899,32 @@ bool YateH323EndPoint::internalGkClient(int mode, const PString& name)
 	case Unregister:
 	    RemoveGatekeeper();
 	    Debug(&hplugin,DebugInfo,"Removed the GK in '%s'",safe());
+	    internalGkNotify(false);
 	    return true;
     }
+    internalGkNotify(false);
     return false;
 }
 
+void YateH323EndPoint::internalGkNotify(bool registered)
+{
+    if ((m_registered == registered) || null())
+	return;
+    m_registered = registered;
+    Message* m = new Message("user.notify");
+    m->addParam("account",*this);
+    m->addParam("protocol","h323");
+    m->addParam("registered",String::boolText(registered));
+    Engine::enqueue(m);
+}
+
+void YateH323EndPoint::checkGkClient()
+{
+    if (!m_thread)
+	internalGkNotify(IsRegisteredWithGatekeeper());
+}
+
+
 YateH323Connection::YateH323Connection(YateH323EndPoint& endpoint,
     H323Transport* transport, unsigned callReference, void* userdata)
     : H323Connection(endpoint,callReference), m_chan(0), m_mutex(0),
@@ -924,6 +957,7 @@ YateH323Connection::YateH323Connection(YateH323EndPoint& endpoint,
 
     CallEndpoint* ch = YOBJECT(CallEndpoint,msg->userData());
     if (ch && ch->connect(m_chan,msg->getValue("reason"))) {
+	m_chan->callConnect(*msg);
 	m_chan->setTarget(msg->getValue("id"));
 	msg->setParam("peerid",m_chan->id());
 	msg->setParam("targetid",m_chan->id());
@@ -1112,6 +1146,8 @@ void YateH323Connection::OnEstablished()
     Debug(this,DebugInfo,"YateH323Connection::OnEstablished() [%p]",this);
     if (!m_chan)
 	return;
+    if (m_chan->address().null())
+	m_chan->setAddress(GetControlChannel().GetRemoteAddress());
     if (HadAnsweredCall()) {
 	m_chan->status("connected");
 	return;
@@ -1848,9 +1884,7 @@ YateH323Chan::YateH323Chan(YateH323Connection* conn,Message* msg,const char* add
     s_mutex.lock();
     s_chanCount++;
     s_mutex.unlock();
-    m_address = addr;
-    m_address.startSkip("ip$",false);
-    filterDebug(m_address);
+    setAddress(addr);
     Debug(this,DebugAll,"YateH323Chan::YateH323Chan(%p,%s) %s [%p]",
 	conn,addr,direction(),this);
     setMaxcall(msg);
@@ -1946,6 +1980,14 @@ void YateH323Chan::disconnected(bool final, const char *reason)
 	m_conn->ClearCall((H323Connection::CallEndReason)lookup(reason,dict_errors,H323Connection::EndedByLocalUser));
 }
 
+// Set the signalling address
+void YateH323Chan::setAddress(const char* addr)
+{
+    m_address = addr;
+    m_address.startSkip("ip$",false);
+    filterDebug(m_address);
+}
+
 // Shut down the data transfers so OpenH323 can stop its related threads
 bool YateH323Chan::stopDataLinks()
 {
@@ -2018,6 +2060,9 @@ void YateH323Chan::callAccept(Message& msg)
 void YateH323Chan::callRejected(const char* error, const char* reason, const Message* msg)
 {
     Channel::callRejected(error,reason,msg);
+    stopDataLinks();
+    if (m_conn)
+	m_conn->ClearCall((H323Connection::CallEndReason)lookup(error,dict_errors,H323Connection::EndedByLocalUser));
 }
 
 bool YateH323Chan::msgProgress(Message& msg)
@@ -2182,6 +2227,14 @@ bool H323Driver::msgExecute(Message& msg, String& dest)
     msg.setParam("error","offline");
     return false;
 };
+
+void H323Driver::msgTimer(Message& msg)
+{
+    Driver::msgTimer(msg);
+    ObjList* l = m_endpoints.skipNull();
+    for (; l; l = l->skipNext())
+	static_cast<YateH323EndPoint*>(l->get())->checkGkClient();
+}
 		    
 YateH323EndPoint* H323Driver::findEndpoint(const String& ep) const
 {
@@ -2195,7 +2248,7 @@ void H323Driver::initialize()
     s_cfg = Engine::configFile("h323chan");
     s_cfg.load();
     setup();
-    s_externalRtp = s_cfg.getBoolValue("general","external_rtp",false);
+    s_externalRtp = s_cfg.getBoolValue("general","external_rtp",true);
     s_passtrough = s_cfg.getBoolValue("general","forward_rtp",false);
     s_fallbackRtp = s_cfg.getBoolValue("general","fallback_rtp",true);
     // mantain compatibility with old config files
