@@ -87,6 +87,7 @@ public:
     inline int remotePort() const
 	{ return m_remotePort; }
 private:
+    void setRegistered(bool registered, const char* reason = 0);
     State m_state;
     String m_username;                  // Username
     String m_password;                  // Password
@@ -115,8 +116,9 @@ public:
 
     /*
      * Logout and remove all lines
+     * @param forced Forcedly remove lines synchronously
      */
-    void clear();
+    void clear(bool forced = false);
 
     /*
      * Update a line from a message
@@ -219,6 +221,7 @@ class YIAXEngine : public IAXEngine
 public:
     /*
      * Constructor
+     * @param iface Interface address to use
      * @param port UDP port to use
      * @param transListCount Number of entries in the transaction hash table
      * @param retransCount Retransmission counter for each transaction belonging to this engine
@@ -228,7 +231,7 @@ public:
      * @param maxFullFrameDataLen Max full frame IE list (buffer) length
      * @param trunkSendInterval Send trunk meta frame interval
      */
-    YIAXEngine(int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
+    YIAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
 	u_int16_t authTimeout, u_int16_t transTimeout,
 	u_int16_t maxFullFrameDataLen, u_int32_t trunkSendInterval);
 
@@ -320,27 +323,27 @@ public:
     virtual bool msgRoute(Message& msg);
     virtual bool received(Message& msg, int id);
 
+    // Default codec map - should have only one bit set
     inline u_int32_t defaultCodec() const
 	{ return m_defaultCodec; }
 
+    // Supported codec map
     inline u_int32_t codecs() const
 	{ return m_codecs; }
 
+    // UDP port we are using, also used as default
     inline int port() const
 	{ return m_port; }
 
+    // Retrive the IAX engine
     inline YIAXEngine* getEngine() const
 	{ return m_iaxEngine; }
 
-    static bool msgHasTrunkIn(Message& msg);    // Test param: trunkin=yes
-
-    static bool msgHasTrunkOut(Message& msg);   // Test param: trunkout=yes
-
 protected:
     YIAXEngine* m_iaxEngine;
     u_int32_t m_defaultCodec;
     u_int32_t m_codecs;
-    int m_port;				// Default UDP port
+    int m_port;
 };
 
 class YIAXConnection;
@@ -357,7 +360,7 @@ public:
 private:
     YIAXConnection* m_connection;
     unsigned m_total;
-    u_int32_t m_format;
+    u_int32_t m_format; // in IAX coding
 };
 
 /*
@@ -372,7 +375,7 @@ public:
 private:
     YIAXConnection* m_connection;
     unsigned m_total;
-    u_int32_t m_format;
+    u_int32_t m_format; // in IAX coding
 };
 
 /*
@@ -432,7 +435,7 @@ private:
 };
 
 /*
- * IAXURI
+ * An IAX URI parser
  *  [iax[2]:][username@]host[:port][/called_number[@called_context]]
  */
 class IAXURI : public String
@@ -478,9 +481,7 @@ static YIAXLineContainer s_lines;	// Lines
  * Class definitions
  */
 
-/*
- * YIAXLine
- */
+// Create an idle line
 YIAXLine::YIAXLine(const String& name)
     : String(name),
       m_state(Idle), m_expire(60), m_localPort(4569), m_remotePort(4569),
@@ -494,9 +495,25 @@ YIAXLine::~YIAXLine()
 {
 }
 
-/*
- * YIAXLineContainer
- */
+// Set the registered status, emits user.notify messages if necessary
+void YIAXLine::setRegistered(bool registered, const char* reason)
+{
+    if ((m_registered == registered) && !reason)
+	return;
+    m_registered = registered;
+    if (m_username) {
+	Message* m = new Message("user.notify");
+	m->addParam("account",*this);
+	m->addParam("protocol","iax");
+	m->addParam("username",m_username);
+	m->addParam("registered",String::boolText(registered));
+	if (reason)
+	    m->addParam("reason",reason);
+	Engine::enqueue(m);
+    }
+}
+
+// Find and update a line with parameters from message, create if needed
 bool YIAXLineContainer::updateLine(Message& msg)
 {
     Lock lock(this);
@@ -507,6 +524,7 @@ bool YIAXLineContainer::updateLine(Message& msg)
     return addLine(msg);
 }
 
+// Handle registration related transaction terminations
 void YIAXLineContainer::regTerminate(IAXEvent* event)
 {
     Lock lock(this);
@@ -526,21 +544,21 @@ void YIAXLineContainer::regTerminate(IAXEvent* event)
 	    line->m_callingName = trans->callingName();
 	    Debug(&iplugin,DebugAll,"YIAXLineContainer::regTerminate[%s] - ACK for '%s'. Next: %u",
 		line->c_str(),line->state() == YIAXLine::Registering?"Register":"Unregister",line->m_nextReg);
-	    line->m_registered = true;
+	    line->setRegistered(true);
 	    break;
 	case IAXEvent::Reject:
 	    // retry at 25% of the expire time
 	    line->m_nextReg = Time::secNow() + (line->expire() / 2);
 	    Debug(&iplugin,DebugAll,"YIAXLineContainer::regTerminate[%s] - REJECT for '%s'. Next: %u",
 		line->c_str(),line->state() == YIAXLine::Registering?"Register":"Unregister",line->m_nextReg);
-	    line->m_registered = false;
+	    line->setRegistered(false,"rejected");
 	    break;
 	case IAXEvent::Timeout:
 	    // retry at 50% of the expire time
 	    line->m_nextReg = Time::secNow() + (line->expire() / 2);
 	    Debug(&iplugin,DebugAll,"YIAXLineContainer::regTerminate[%s] - Timeout for '%s'. Next: %u",
 		line->c_str(),line->state() == YIAXLine::Registering?"Register":"Unregister",line->m_nextReg);
-	    line->m_registered = false;
+	    line->setRegistered(false,"timeout");
 	    break;
 	default:
 	    return;
@@ -548,12 +566,14 @@ void YIAXLineContainer::regTerminate(IAXEvent* event)
     line->m_transaction = 0;
     // Unregister operation. Remove line
     if (line->state() == YIAXLine::Unregistering) {
+	line->setRegistered(false);
 	m_lines.remove(line,true);
 	return;
     }
     line->m_state = YIAXLine::Idle;
 }
 
+// Handle registration related events
 void YIAXLineContainer::handleEvent(IAXEvent* event)
 {
     switch (event->type()) {
@@ -567,6 +587,7 @@ void YIAXLineContainer::handleEvent(IAXEvent* event)
     }
 }
 
+// Tick the timer for all lines, send keepalives and reregister
 void YIAXLineContainer::evTimer(Time& time)
 {
     u_int32_t sec = time.sec();
@@ -595,6 +616,7 @@ void YIAXLineContainer::evTimer(Time& time)
     }
 }
 
+// Fill parameters with information taken from line
 bool YIAXLineContainer::fillList(String& name, NamedList& dest, SocketAddr& addr, bool& registered)
 {
     Lock lock(this);
@@ -612,6 +634,7 @@ bool YIAXLineContainer::fillList(String& name, NamedList& dest, SocketAddr& addr
     return true;
 }
 
+// Update a line with data from message, create or delete line if needed
 bool YIAXLineContainer::updateLine(YIAXLine* line, Message& msg)
 {
     Debug(&iplugin,DebugAll,"YIAXLineContainer - updateLine: %s",line->c_str());
@@ -648,6 +671,7 @@ bool YIAXLineContainer::updateLine(YIAXLine* line, Message& msg)
     return change;
 }
 
+// Add a new line and start logging in to it if applicable
 bool YIAXLineContainer::addLine(Message& msg)
 {
     Debug(&iplugin,DebugAll,"YIAXLineContainer - addLine: %s",msg.getValue("account"));
@@ -669,6 +693,7 @@ bool YIAXLineContainer::addLine(Message& msg)
     return true;
 }
 
+// Find a line by name
 YIAXLine* YIAXLineContainer::findLine(const String& name)
 {
     for (ObjList* l = m_lines.skipNull(); l; l = l->next()) {
@@ -679,6 +704,7 @@ YIAXLine* YIAXLineContainer::findLine(const String& name)
     return 0;
 }
 
+// Find a line by address, return same if found
 YIAXLine* YIAXLineContainer::findLine(YIAXLine* line)
 {
     if (!line)
@@ -691,6 +717,7 @@ YIAXLine* YIAXLineContainer::findLine(YIAXLine* line)
     return 0;
 }
 
+// Initiate registration of line, only if idle
 void YIAXLineContainer::startRegisterLine(YIAXLine* line)
 {
     Lock lock(this);
@@ -701,6 +728,7 @@ void YIAXLineContainer::startRegisterLine(YIAXLine* line)
 	line->m_state = YIAXLine::Registering;
 }
 
+// Initiate deregistration of line, only if idle
 void YIAXLineContainer::startUnregisterLine(YIAXLine* line)
 {
     Lock lock(this);
@@ -711,9 +739,14 @@ void YIAXLineContainer::startUnregisterLine(YIAXLine* line)
 	line->m_state = YIAXLine::Unregistering;
 }
 
-void YIAXLineContainer::clear()
+// Unregister all lines
+void YIAXLineContainer::clear(bool forced)
 {
     Lock lock(this);
+    if (forced) {
+	m_lines.clear();
+	return;
+    }
     for (ObjList* l = m_lines.skipNull(); l; l = l->next()) {
 	YIAXLine* line = static_cast<YIAXLine*>(l->get());
 	if (line)
@@ -721,9 +754,7 @@ void YIAXLineContainer::clear()
     }
 }
 
-/**
- * IAXListener
- */
+// Run the socket listening thread
 void YIAXListener::run()
 {
     Debug(m_engine,DebugAll,"%s started",currentName());
@@ -731,18 +762,14 @@ void YIAXListener::run()
     m_engine->readSocket(addr);
 }
 
-/**
- * IAXGetEvent
- */
+// Run the event retriving thread
 void YIAXGetEvent::run()
 {
     Debug(m_engine,DebugAll,"%s started",currentName());
     m_engine->runGetEvents();
 }
 
-/**
- * YIAXTrunking
- */
+// Run the trunk sending thread
 void YIAXTrunking::run()
 {
     Debug(m_engine,DebugAll,"%s started",currentName());
@@ -752,14 +779,15 @@ void YIAXTrunking::run()
 /**
  * YIAXEngine
  */
-YIAXEngine::YIAXEngine(int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
+YIAXEngine::YIAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
 	u_int16_t authTimeout, u_int16_t transTimeout, u_int16_t maxFullFrameDataLen, u_int32_t trunkSendInterval)
-    : IAXEngine(port,transListCount,retransCount,retransInterval,authTimeout,transTimeout,
+    : IAXEngine(iface,port,transListCount,retransCount,retransInterval,authTimeout,transTimeout,
       maxFullFrameDataLen,iplugin.defaultCodec(),iplugin.codecs(),trunkSendInterval),
       m_threadsCreated(false)
 {
 }
 
+// Handle received voice data, forward it to connection's source
 void YIAXEngine::processMedia(IAXTransaction* transaction, DataBlock& data, u_int32_t tStamp)
 {
     if (transaction)
@@ -775,6 +803,7 @@ void YIAXEngine::processMedia(IAXTransaction* transaction, DataBlock& data, u_in
 	Debug(this,DebugAll,"YIAXEngine - processMedia. No transaction");
 }
 
+// Create a new registration transaction for a line
 IAXTransaction* YIAXEngine::reg(YIAXLine* line, bool regreq)
 {
     if (!line)
@@ -799,6 +828,7 @@ IAXTransaction* YIAXEngine::reg(YIAXLine* line, bool regreq)
     return tr;
 }
 
+// Create a new call transaction from target address and message params
 IAXTransaction* YIAXEngine::call(SocketAddr& addr, NamedList& params)
 {
     Debug(this,DebugAll,"Outgoing Call:\nUsername: %s\nHost: %s\nPort: %d\nCalled number: %s\nCalled context: %s",
@@ -810,12 +840,13 @@ IAXTransaction* YIAXEngine::call(SocketAddr& addr, NamedList& params)
     ieList.appendString(IAXInfoElement::CALLING_NUMBER,params.getValue("caller"));
     ieList.appendString(IAXInfoElement::CALLING_NAME,params.getValue("callername"));
     ieList.appendString(IAXInfoElement::CALLED_NUMBER,params.getValue("called"));
-    ieList.appendString(IAXInfoElement::CALLED_CONTEXT,params.getValue("calledname"));
+    ieList.appendString(IAXInfoElement::CALLED_CONTEXT,params.getValue("iaxcontext"));
     ieList.appendNumeric(IAXInfoElement::FORMAT,iplugin.defaultCodec(),4);
     ieList.appendNumeric(IAXInfoElement::CAPABILITY,iplugin.codecs(),4);
     return startLocalTransaction(IAXTransaction::New,addr,ieList);
 }
 
+// Create a POKE transaction
 IAXTransaction* YIAXEngine::poke(SocketAddr& addr)
 {
     Debug(this,DebugAll,"Outgoing POKE: Host: %s Port: %d",addr.host().c_str(),addr.port());
@@ -828,11 +859,11 @@ void YIAXEngine::start(u_int16_t listenThreadCount, u_int16_t eventThreadCount,
     if (m_threadsCreated)
 	return;
     if (!listenThreadCount)
-	Debug(DebugWarn,"YIAXEngine. No reading socket threads(s)!.");
+	Debug(this,DebugWarn,"YIAXEngine. No reading socket threads(s)!.");
     if (!eventThreadCount)
-	Debug(DebugWarn,"YIAXEngine. No reading event threads(s)!.");
+	Debug(this,DebugWarn,"YIAXEngine. No reading event threads(s)!.");
     if (!trunkThreadCount)
-	Debug(DebugWarn,"YIAXEngine. No trunking threads(s)!.");
+	Debug(this,DebugWarn,"YIAXEngine. No trunking threads(s)!.");
     for (; listenThreadCount; listenThreadCount--)
 	(new YIAXListener(this,"YIAXListener thread"))->startup();
     for (; eventThreadCount; eventThreadCount--)
@@ -842,6 +873,7 @@ void YIAXEngine::start(u_int16_t listenThreadCount, u_int16_t eventThreadCount,
     m_threadsCreated = true;
 }
 
+// Process all IAX events
 void YIAXEngine::processEvent(IAXEvent* event)
 {
     YIAXConnection* connection = 0;
@@ -880,6 +912,7 @@ void YIAXEngine::processEvent(IAXEvent* event)
     delete event;
 }
 
+// Process events for remote users registering to us
 void YIAXEngine::processRemoteReg(IAXEvent* event)
 {
     IAXTransaction* tr = event->getTransaction();
@@ -897,7 +930,7 @@ void YIAXEngine::processRemoteReg(IAXEvent* event)
 	return;
     }
     String password = msg.retValue();
-    if (!password.length()) {
+    if (password.null()) {
 	// Authenticated, no password. Try to (un)register
 	if (userreg(tr,event->subclass() == IAXControl::RegRel)) {
 	    Debug(this,DebugAll,"processRemoteReg. Authenticated and (un)registered. Ack");
@@ -914,6 +947,7 @@ void YIAXEngine::processRemoteReg(IAXEvent* event)
     tr->sendAuth(password);
 }
 
+// Build and dispatch the user.(un)register message
 bool YIAXEngine::userreg(IAXTransaction* tr, bool regrel)
 {
     Debug(this,DebugAll,"YIAXEngine - userreg. %s username: '%s'",
@@ -935,9 +969,7 @@ bool YIAXEngine::userreg(IAXTransaction* tr, bool regrel)
     return Engine::dispatch(msg);
 }
 
-/**
- * YIAXRegDataHandler
- */
+// Handler for outgoing registration messages
 bool YIAXRegDataHandler::received(Message& msg)
 {
     String tmp(msg.getValue("protocol"));
@@ -950,9 +982,6 @@ bool YIAXRegDataHandler::received(Message& msg)
     return true;
 }
 
-/**
- * YIAXDriver
- */
 YIAXDriver::YIAXDriver()
     : Driver("iax","varchans"), m_iaxEngine(0), m_port(4569)
 {
@@ -964,6 +993,7 @@ YIAXDriver::~YIAXDriver()
     Output("Unloading module YIAX");
     lock();
     channels().clear();
+    s_lines.clear(true);
     unlock();
     delete m_iaxEngine;
 }
@@ -996,8 +1026,9 @@ void YIAXDriver::initialize()
     // If desired codec is disabled fall back to last in list
     if (!m_defaultCodec)
 	m_defaultCodec = fallback;
-    // Port
+    // Port and interface
     m_port = s_cfg.getIntValue("general","port",4569);
+    String iface = s_cfg.getValue("general","addr");
     unlock();
     setup();
     // We need channels to be dropped on shutdown
@@ -1013,7 +1044,7 @@ void YIAXDriver::initialize()
     u_int32_t trunkSendInterval = 10;
     if (!m_iaxEngine) {
 	Engine::install(new YIAXRegDataHandler);
-	m_iaxEngine = new YIAXEngine(m_port,transListCount,retransCount,retransInterval,authTimeout,
+	m_iaxEngine = new YIAXEngine(iface,m_port,transListCount,retransCount,retransInterval,authTimeout,
 		transTimeout,maxFullFrameDataLen,trunkSendInterval);
 	m_iaxEngine->debugChain(this);
 	int tos = s_cfg.getIntValue("general","tos",dict_tos,0);
@@ -1028,6 +1059,7 @@ void YIAXDriver::initialize()
     m_iaxEngine->start(readThreadCount,eventThreadCount,trunkingThreadCount);
 }
 
+// Route calls that use a line owned by this driver
 bool YIAXDriver::msgRoute(Message& msg)
 {
     String called = msg.getValue("called");
@@ -1086,10 +1118,11 @@ bool YIAXDriver::msgExecute(Message& msg, String& dest)
     tr->setUserData(conn);
     Channel* ch = static_cast<Channel*>(msg.userData());
     if (ch && conn->connect(ch,msg.getValue("reason"))) {
+	conn->callConnect(msg);
 	msg.setParam("peerid",conn->id());
 	msg.setParam("targetid",conn->id());
-	// Enable trunking
-	if (msgHasTrunkOut(msg))
+	// Enable trunking if trunkout parameter is enabled
+	if (msg.getBoolValue("trunkout"))
 	    m_iaxEngine->enableTrunking(tr);
     }
     else
@@ -1111,16 +1144,6 @@ bool YIAXDriver::received(Message& msg, int id)
     return Driver::received(msg,id);
 }
 
-bool YIAXDriver::msgHasTrunkIn(Message& msg)
-{
-    return String(msg.getValue("trunkin")) == "yes";
-}
-
-bool YIAXDriver::msgHasTrunkOut(Message& msg)
-{
-    return String(msg.getValue("trunkout")) == "yes";
-}
-
 /**
  * IAXConsumer
  */
@@ -1195,20 +1218,22 @@ YIAXConnection::~YIAXConnection()
     hangup();
 }
 
+// Incoming call accepted, possibly set trunking on this connection
 void YIAXConnection::callAccept(Message& msg)
 {
     DDebug(this,DebugAll,"callAccept [%p]",this);
     m_mutexTrans.lock();
     if (m_transaction) {
 	m_transaction->sendAccept();
-	// Enable trunking
-	if (YIAXDriver::msgHasTrunkIn(msg))
+	// Enable trunking if trunkin parameter is enabled
+	if (msg.getBoolValue("trunkin"))
 	    m_iaxEngine->enableTrunking(m_transaction);
     }
     m_mutexTrans.unlock();
     Channel::callAccept(msg);
 }
 
+// Call rejected, check if we have to authenticate caller
 void YIAXConnection::callRejected(const char* error, const char* reason, const Message* msg)
 {
     Channel::callRejected(error,reason,msg);
@@ -1232,6 +1257,7 @@ void YIAXConnection::callRejected(const char* error, const char* reason, const M
 
 bool YIAXConnection::callRouted(Message& msg)
 {
+    // check if the caller did abort the call while routing
     if (!m_transaction) {
 	Debug(this,DebugMild,"callRouted [%p]. No transaction: ABORT",this);
 	return false;
@@ -1245,6 +1271,7 @@ bool YIAXConnection::msgRinging(Message& msg)
     Lock lock(&m_mutexTrans);
     if (m_transaction) {
 	m_transaction->sendRinging();
+	// only start audio output for early media
 	startAudioOut();
 	return Channel::msgRinging(msg);
     }
@@ -1256,6 +1283,7 @@ bool YIAXConnection::msgAnswered(Message& msg)
     Lock lock(&m_mutexTrans);
     if (m_transaction) {
 	m_transaction->sendAnswer();
+	// fully start audio
 	startAudioIn();
 	startAudioOut();
 	return Channel::msgAnswered(msg);
@@ -1293,6 +1321,7 @@ void YIAXConnection::disconnected(bool final, const char* reason)
 
 bool YIAXConnection::callPrerouted(Message& msg, bool handled)
 {
+    // check if the caller did abort the call while prerouting
     if (!m_transaction) {
 	Debug(this,DebugMild,"callPrerouted [%p]. No transaction: ABORT",this);
 	return false;
@@ -1437,6 +1466,7 @@ bool YIAXConnection::route(bool authenticated)
     return startRouter(m);
 }
 
+// Create audio source with the proper format
 void YIAXConnection::startAudioIn()
 {
     if (getSource())
@@ -1452,6 +1482,7 @@ void YIAXConnection::startAudioIn()
     DDebug(this,DebugAll,"startAudioIn. Format %u: '%s'",format,formatText);
 }
 
+// Create audio consumer with the proper format
 void YIAXConnection::startAudioOut()
 {
     if (getConsumer())
@@ -1495,6 +1526,7 @@ void YIAXConnection::evAuthRep(IAXEvent* event)
     route(true);
 }
 
+// Get rid of the extra reference
 void YIAXConnection::safeDeref()
 {
     m_mutexRefIncreased.lock();
@@ -1505,6 +1537,7 @@ void YIAXConnection::safeDeref()
 	deref();
 }
 
+// Keep an extra reference to prevent destroying the connection
 bool YIAXConnection::safeRefIncrease()
 {
     bool ok = false;
@@ -1515,9 +1548,7 @@ bool YIAXConnection::safeRefIncrease()
     return ok;
 }
 
-/**
- * IAXURI
- */
+// IAX URI constructor from components
 IAXURI::IAXURI(const char* user, const char* host, const char* calledNo, const char* calledContext, int port)
     : m_username(user),
       m_host(host),
@@ -1542,10 +1573,10 @@ IAXURI::IAXURI(const char* user, const char* host, const char* calledNo, const c
 
 void IAXURI::parse()
 {
-#if 0
-proto: user@ host :port /calledno @context
-proto: user@ host :port /calledno ?context
-#endif
+/*
+    proto: user@ host :port /calledno @context
+    proto: user@ host :port /calledno ?context
+*/
     if (m_parsed)
 	return;
     String tmp(*this), _port;
@@ -1573,6 +1604,7 @@ proto: user@ host :port /calledno ?context
     m_parsed = true;
 }
 
+// Pick URI parameters from a message or setting
 bool IAXURI::fillList(NamedList& dest)
 {
     if (!m_parsed)
@@ -1582,7 +1614,7 @@ bool IAXURI::fillList(NamedList& dest)
     if (m_calledNo.length())
 	dest.setParam("called",m_calledNo);
     if (m_calledContext.length())
-	dest.setParam("calledname",m_calledContext);
+	dest.setParam("iaxcontext",m_calledContext);
     return true;
 }
 
