@@ -32,6 +32,7 @@
 #define MIN_PORT 16384
 #define MAX_PORT 32768
 #define BUF_SIZE 240
+#define BUF_PREF 160
 
 using namespace TelEngine;
 namespace { // anonymous
@@ -91,9 +92,9 @@ class YRTPWrapper : public RefObject
     friend class YRTPConsumer;
     friend class YRTPSession;
 public:
-    YRTPWrapper(const char *localip, CallEndpoint* conn = 0, const char* media = "audio", RTPSession::Direction direction = RTPSession::SendRecv);
+    YRTPWrapper(const char *localip, CallEndpoint* conn = 0, const char* media = "audio", RTPSession::Direction direction = RTPSession::SendRecv, bool rtcp = true);
     ~YRTPWrapper();
-    void setupRTP(const char* localip);
+    void setupRTP(const char* localip, bool rtcp);
     bool startRTP(const char* raddr, unsigned int rport, const Message& msg);
     bool sendDTMF(char dtmf, int duration = 0);
     void gotDTMF(char tone);
@@ -175,8 +176,11 @@ public:
     YRTPConsumer(YRTPWrapper* wrap);
     ~YRTPConsumer();
     virtual void Consume(const DataBlock &data, unsigned long tStamp);
+    inline void setSplitable()
+	{ m_splitable = (m_format == "alaw") || (m_format == "mulaw"); }
 private:
     YRTPWrapper* m_wrap;
+    bool m_splitable;
 };
 
 class AttachHandler : public MessageHandler
@@ -216,13 +220,13 @@ static ObjList s_calls;
 static Mutex s_mutex;
 static Mutex s_srcMutex;
 
-YRTPWrapper::YRTPWrapper(const char* localip, CallEndpoint* conn, const char* media, RTPSession::Direction direction)
+YRTPWrapper::YRTPWrapper(const char* localip, CallEndpoint* conn, const char* media, RTPSession::Direction direction, bool rtcp)
     : m_rtp(0), m_dir(direction), m_conn(conn),
       m_source(0), m_consumer(0), m_media(media),
       m_bufsize(0), m_port(0)
 {
-    Debug(&splugin,DebugAll,"YRTPWrapper::YRTPWrapper('%s',%p,'%s',%s) [%p]",
-	localip,conn,media,lookup(direction,dict_yrtp_dir),this);
+    Debug(&splugin,DebugAll,"YRTPWrapper::YRTPWrapper('%s',%p,'%s',%s,%s) [%p]",
+	localip,conn,media,lookup(direction,dict_yrtp_dir),String::boolText(rtcp),this);
     m_id = "yrtp/";
     m_id << (unsigned int)::random();
     if (conn)
@@ -230,7 +234,7 @@ YRTPWrapper::YRTPWrapper(const char* localip, CallEndpoint* conn, const char* me
     m_audio = (m_media == "audio");
     s_mutex.lock();
     s_calls.append(this);
-    setupRTP(localip);
+    setupRTP(localip,rtcp);
     s_mutex.unlock();
 }
 
@@ -283,7 +287,7 @@ YRTPWrapper* YRTPWrapper::find(const String& id)
     return 0;
 }
 
-void YRTPWrapper::setupRTP(const char* localip)
+void YRTPWrapper::setupRTP(const char* localip, bool rtcp)
 {
     Debug(&splugin,DebugAll,"YRTPWrapper::setupRTP(\"%s\") [%p]",localip,this);
     m_rtp = new YRTPSession(this);
@@ -302,15 +306,15 @@ void YRTPWrapper::setupRTP(const char* localip)
     }
     SocketAddr addr(AF_INET);
     if (!addr.host(localip)) {
-	Debug(&splugin,DebugWarn,"YRTPWrapper [%p] could not parse address '%s'",this,localip);
+	Debug(&splugin,DebugWarn,"Wrapper could not parse address '%s' [%p]",localip,this);
 	return;
     }
     for (; attempt; attempt--) {
 	int lport = (minport + (::random() % (maxport - minport))) & 0xfffe;
 	addr.port(lport);
-	if (m_rtp->localAddr(addr)) {
+	if (m_rtp->localAddr(addr,rtcp)) {
 	    m_port = lport;
-	    Debug(&splugin,DebugAll,"YRTPWrapper [%p] RTP %p bound to %s:%u",this,m_rtp,localip,m_port);
+	    Debug(&splugin,DebugAll,"Session %p bound to %s:%u [%p]",m_rtp,localip,m_port,this);
 	    return;
 	}
     }
@@ -321,12 +325,12 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, const Message&
 {
     Debug(&splugin,DebugAll,"YRTPWrapper::startRTP(\"%s\",%u) [%p]",raddr,rport,this);
     if (!m_rtp) {
-	Debug(&splugin,DebugWarn,"YRTPWrapper [%p] attempted to start RTP before setup!",this);
+	Debug(&splugin,DebugWarn,"Wrapper attempted to start RTP before setup! [%p]",this);
 	return false;
     }
 
     if (m_bufsize) {
-	DDebug(&splugin,DebugAll,"YRTPWrapper [%p] attempted to restart RTP!",this);
+	DDebug(&splugin,DebugAll,"Wrapper attempted to restart RTP! [%p]",this);
 	m_rtp->resync();
 	return true;
     }
@@ -343,19 +347,22 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, const Message&
     if (!format)
 	format = lookup(payload, dict_payloads);
     if (!format) {
-	Debug(&splugin,DebugWarn,"YRTPWrapper [%p] can't find name for payload %d",this,payload);
+	if (payload < 0)
+	    Debug(&splugin,DebugWarn,"Wrapper neither format nor payload specified [%p]",this);
+	else
+	    Debug(&splugin,DebugWarn,"Wrapper can't find name for payload %d [%p]",payload,this);
 	return false;
     }
 
     if (payload == -1)
 	payload = lookup(format, dict_payloads, -1);
     if (payload == -1) {
-	Debug(&splugin,DebugWarn,"YRTPWrapper [%p] can't find payload for format %s",this,format);
+	Debug(&splugin,DebugWarn,"Wrapper can't find payload for format %s [%p]",format,this);
 	return false;
     }
 
     if ((payload < 0) || (payload >= 127)) {
-	Debug(&splugin,DebugWarn,"YRTPWrapper [%p] received invalid payload %d",this,payload);
+	Debug(&splugin,DebugWarn,"Wrapper received invalid payload %d [%p]",payload,this);
 	return false;
     }
 
@@ -388,6 +395,7 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, const Message&
 	    m_conn->setConsumer(0,m_media);
 	}
 	m_consumer->m_format = format;
+	m_consumer->setSplitable();
 	if (m_conn) {
 	    m_conn->setConsumer(m_consumer,m_media);
 	    m_consumer->deref();
@@ -535,7 +543,7 @@ YRTPSource::~YRTPSource()
 }
 
 YRTPConsumer::YRTPConsumer(YRTPWrapper *wrap)
-    : m_wrap(wrap)
+    : m_wrap(wrap), m_splitable(false)
 {
     Debug(&splugin,DebugAll,"YRTPConsumer::YRTPConsumer(%p) [%p]",wrap,this);
     m_format.clear();
@@ -560,24 +568,24 @@ void YRTPConsumer::Consume(const DataBlock &data, unsigned long tStamp)
 {
     if (!(m_wrap && m_wrap->bufSize() && m_wrap->rtp()))
 	return;
-    unsigned long delta = (tStamp && m_timestamp) ? (tStamp - m_timestamp) : 0;
-    XDebug(&splugin,DebugAll,"YRTPConsumer writing %d bytes, delta=%lu ts=%lu [%p]",
-	data.length(),delta,tStamp,this);
+    XDebug(&splugin,DebugAll,"YRTPConsumer writing %d bytes, ts=%lu [%p]",
+	data.length(),tStamp,this);
     unsigned int buf = m_wrap->bufSize();
     const char* ptr = (const char*)data.data();
     unsigned int len = data.length();
-    // make it safe to break a long octet buffer
-    if (len == delta)
-	delta = 0;
     while (len && m_wrap && m_wrap->rtp()) {
 	unsigned int sz = len;
-	if (m_wrap->isAudio() && (sz > buf) && !delta) {
-	    DDebug(&splugin,DebugAll,"Creating %u bytes fragment of %u bytes buffer",buf,len);
-	    sz = buf;
+	if (m_splitable && m_wrap->isAudio() && (sz > buf)) {
+	    // divide evenly a buffer that is multiple of preferred size
+	    if ((buf > BUF_PREF) && ((len % BUF_PREF) == 0))
+		sz = BUF_PREF;
+	    else
+		sz = buf;
+	    DDebug(&splugin,DebugAll,"Creating %u bytes fragment of %u bytes buffer",sz,len);
 	}
 	m_wrap->rtp()->rtpSendData(false,tStamp,ptr,sz);
 	// if timestamp increment is not provided we have to guess...
-	tStamp += delta ? delta : sz;
+	tStamp += sz;
 	len -= sz;
 	ptr += sz;
     }
@@ -590,11 +598,8 @@ bool AttachHandler::received(Message &msg)
     if (src.null())
 	more--;
     else {
-	Regexp r("^rtp/\\(.*\\)$");
-	if (src.matches(r)) {
-	    src = src.matchString(1);
+	if (src.startSkip("rtp/",false))
 	    more--;
-	}
 	else
 	    src = "";
     }
@@ -603,11 +608,8 @@ bool AttachHandler::received(Message &msg)
     if (cons.null())
 	more--;
     else {
-	Regexp r("^rtp/\\(.*\\)$");
-	if (cons.matches(r)) {
-	    cons = cons.matchString(2);
+	if (cons.startSkip("rtp/",false))
 	    more--;
-	}
 	else
 	    cons = "";
     }
@@ -647,6 +649,9 @@ bool AttachHandler::received(Message &msg)
 	    ch->setConsumer(c,media);
 	    c->deref();
 	}
+
+	if (w->deref())
+	    return false;
     }
 
     if (rip && rport)
@@ -661,6 +666,9 @@ bool AttachHandler::received(Message &msg)
 
 bool RtpHandler::received(Message &msg)
 {
+    String trans = msg.getValue("transport");
+    if (trans && !trans.startsWith("RTP/"))
+	return false;
     Debug(&splugin,DebugAll,"RTP message received");
     String dir(msg.getValue("direction"));
     RTPSession::Direction direction = RTPSession::SendRecv;
@@ -697,11 +705,11 @@ bool RtpHandler::received(Message &msg)
 
     YRTPWrapper *w = YRTPWrapper::find(ch,media);
     if (w)
-	Debug(&splugin,DebugAll,"YRTPWrapper %p found by CallEndpoint",w);
+	Debug(&splugin,DebugAll,"Wrapper %p found by CallEndpoint",w);
     if (!w) {
 	w = YRTPWrapper::find(msg.getValue("rtpid"));
 	if (w)
-	    Debug(&splugin,DebugAll,"YRTPWrapper %p found by ID",w);
+	    Debug(&splugin,DebugAll,"Wrapper %p found by ID",w);
     }
     if (!w) {
 	String lip(msg.getValue("localip"));
@@ -713,7 +721,7 @@ bool RtpHandler::received(Message &msg)
 	}
 	msg.setParam("localip",lip);
 
-	w = new YRTPWrapper(lip,ch,media,direction);
+	w = new YRTPWrapper(lip,ch,media,direction,msg.getBoolValue("rtcp",true));
 	w->setMaster(msg.getValue("id"));
     }
     else {
