@@ -330,6 +330,13 @@ public:
     inline unsigned long timeStamp() const
 	{ return m_timestamp; }
 
+    /**
+     * Get the internal representation of an invalid or unknown timestamp
+     * @return Invalid timestamp - unsigned long conversion of -1
+     */
+    inline static unsigned long invalidStamp()
+	{ return (unsigned long)-1; }
+
 protected:
     DataFormat m_format;
     unsigned long m_timestamp;
@@ -428,7 +435,7 @@ public:
      * @param format Name of the data format, default "slin" (Signed Linear)
      */
     inline DataSource(const char* format = "slin")
-	: DataNode(format), m_translator(0) { }
+	: DataNode(format), m_nextStamp(invalidStamp()), m_translator(0) { }
 
     /**
      * Source's destructor - detaches all consumers
@@ -447,7 +454,7 @@ public:
      * @param data The raw data block to forward; an empty block ends data
      * @param tStamp Timestamp of data - typically samples
      */
-    void Forward(const DataBlock& data, unsigned long tStamp = (unsigned long)-1);
+    void Forward(const DataBlock& data, unsigned long tStamp = invalidStamp());
 
     /**
      * Attach a data consumer
@@ -489,7 +496,15 @@ public:
      */
     void synchronize(unsigned long tStamp);
 
+    /**
+     * Get the next expected position in the data stream
+     * @return Timestamp of next expected data position, may be invalid/unknown
+     */
+    inline unsigned long nextStamp() const
+	{ return m_nextStamp; }
+
 protected:
+    unsigned long m_nextStamp;
     DataTranslator* m_translator;
     ObjList m_consumers;
     Mutex m_mutex;
@@ -884,6 +899,33 @@ public:
     inline DataConsumer* getCallRecord() const
 	{ return m_callRecord; }
 
+    /**
+     * Adds a data consumer to the list of sniffers of the local call data
+     * @param sniffer Pointer to the DataConsumer to add to sniffer list
+     * @return True if the sniffer was added to list, false if NULL or already added
+     */
+    bool addSniffer(DataConsumer* sniffer);
+
+    /**
+     * Remove a data consumer from the list of sniffers of the local call data
+     * @param sniffer Pointer to the DataConsumer to remove from sniffer list
+     * @return True if the sniffer was removed from list
+     */
+    bool delSniffer(DataConsumer* sniffer);
+
+    /**
+     * Find a sniffer by name
+     * @param name Name of the sniffer to find
+     * @return Pointer to DataConsumer or NULL if not found
+     */
+    inline DataConsumer* getSniffer(const String& name)
+	{ return static_cast<DataConsumer*>(m_sniffers[name]); }
+
+    /**
+     * Removes all sniffers from the list and dereferences them
+     */
+    void clearSniffers();
+
     /**
      * Get a pointer to the peer endpoint
      * @return A pointer to the peer endpoint or NULL
@@ -922,6 +964,7 @@ private:
     CallEndpoint* m_call;
     DataConsumer* m_peerRecord;
     DataConsumer* m_callRecord;
+    ObjList m_sniffers;
 };
 
 /**
@@ -1454,6 +1497,13 @@ public:
      */
     virtual void callRejected(const char* error, const char* reason = 0, const Message* msg = 0);
 
+    /**
+     * Common processing after connecting the outgoing call, should be called
+     *  from Driver's msgExecute()
+     * @param msg Notification call.execute message while being dispatched
+     */
+    virtual void callConnect(Message& msg);
+
     /**
      * Set the local debugging level
      * @param msg Debug setting message
@@ -1660,6 +1710,14 @@ protected:
      */
     bool dtmfInband(const char* tone);
 
+    /**
+     * Attempt to install a data sniffer to detect inband tones
+     * Needs a tone detector module capable of attaching sniffer consumers.
+     * @param sniffer Name of the sniffer to install
+     * @return True on success
+     */
+    bool toneDetect(const char* sniffer = "tone/*");
+
 private:
     void init();
     Channel(); // no default constructor please
